@@ -54,8 +54,6 @@ logger = logging.getLogger(__name__)
 dd_capture_lambda_payload_enabled = (
     os.environ.get("DD_CAPTURE_LAMBDA_PAYLOAD", "false").lower() == "true"
 )
-service_env_var = os.environ.get("DD_SERVICE", "DefaultServiceName")
-env_env_var = os.environ.get("DD_ENV", None)
 
 DD_FLUSH_TO_LOG = "DD_FLUSH_TO_LOG"
 DD_LOGS_INJECTION = "DD_LOGS_INJECTION"
@@ -70,6 +68,9 @@ DD_MIN_COLD_START_DURATION = "DD_MIN_COLD_START_DURATION"
 DD_COLD_START_TRACE_SKIP_LIB = "DD_COLD_START_TRACE_SKIP_LIB"
 DD_REQUESTS_SERVICE_NAME = "DD_REQUESTS_SERVICE_NAME"
 DD_SERVICE = "DD_SERVICE"
+DD_ENV = "DD_ENV"
+
+env_env_var = os.environ.get(DD_ENV, None)
 
 """
 Usage:
@@ -132,6 +133,7 @@ class _LambdaDecorator(object):
                 os.environ.get(DD_MERGE_XRAY_TRACES, "false").lower() == "true"
             )
             self.function_name = os.environ.get(AWS_LAMBDA_FUNCTION_NAME, "function")
+            self.service = os.environ.get(DD_SERVICE, None)
             self.extractor_env = os.environ.get(DD_TRACE_EXTRACTOR, None)
             self.trace_extractor = None
             self.span = None
@@ -172,7 +174,7 @@ class _LambdaDecorator(object):
                     logger.debug(f"Malformatted for env {DD_COLD_START_TRACE_SKIP_LIB}")
             self.response = None
             if profiling_env_var:
-                self.prof = profiler.Profiler(env=env_env_var, service=service_env_var)
+                self.prof = profiler.Profiler(env=env_env_var, service=self.service)
             if self.extractor_env:
                 extractor_parts = self.extractor_env.rsplit(".", 1)
                 if len(extractor_parts) == 2:
@@ -315,6 +317,9 @@ class _LambdaDecorator(object):
                 if status_code:
                     self.inferred_span.set_tag("http.status_code", status_code)
 
+                if self.service:
+                    self.inferred_span.set_tag("peer.service", self.service)
+
                 if InferredSpanInfo.is_async(self.inferred_span) and self.span:
                     self.inferred_span.finish(finish_time=self.span.start)
                 else:
